@@ -479,13 +479,13 @@ update_places (NautilusPlacesSidebar *sidebar)
 	GList *volumes;
 	GVolume *volume;
 	int bookmark_count, index;
-	char *location, *mount_uri, *name, *desktop_path, *last_uri;
+	char *location, *mount_uri, *name, *desktop_path, *last_uri, *identifier;
 	const gchar *path, *bookmark_name;
 	GIcon *icon;
 	GFile *root;
 	NautilusWindowSlot *slot;
 	char *tooltip;
-	GList *network_mounts;
+	GList *network_mounts, *network_volumes;
 	NautilusFile *file;
 
 	DEBUG ("Updating places sidebar");
@@ -507,6 +507,7 @@ update_places (NautilusPlacesSidebar *sidebar)
 	slot = nautilus_window_get_active_slot (sidebar->window);
 	location = nautilus_window_slot_get_current_uri (slot);
 
+	network_mounts = network_volumes = NULL;
 	volume_monitor = sidebar->volume_monitor;
 
 	/* first go through all connected drives */
@@ -519,6 +520,15 @@ update_places (NautilusPlacesSidebar *sidebar)
 		if (volumes != NULL) {
 			for (ll = volumes; ll != NULL; ll = ll->next) {
 				volume = ll->data;
+				identifier = g_volume_get_identifier (volume, G_VOLUME_IDENTIFIER_KIND_CLASS);
+
+				if (g_strcmp0 (identifier, "network") == 0) {
+					g_free (identifier);
+					network_volumes = g_list_prepend (network_volumes, volume);
+					continue;
+				}
+				g_free (identifier);
+
 				mount = g_volume_get_mount (volume);
 				if (mount != NULL) {
 					/* Show mounted volume in the sidebar */
@@ -599,6 +609,16 @@ update_places (NautilusPlacesSidebar *sidebar)
 			g_object_unref (drive);
 			continue;
 		}
+
+		identifier = g_volume_get_identifier (volume, G_VOLUME_IDENTIFIER_KIND_CLASS);
+
+		if (g_strcmp0 (identifier, "network") == 0) {
+			g_free (identifier);
+			network_volumes = g_list_prepend (network_volumes, volume);
+			continue;
+		}
+		g_free (identifier);
+
 		mount = g_volume_get_mount (volume);
 		if (mount != NULL) {
 			icon = g_mount_get_icon (mount);
@@ -737,7 +757,6 @@ update_places (NautilusPlacesSidebar *sidebar)
 	}
 
 	/* add mounts that has no volume (/etc/mtab mounts, ftp, sftp,...) */
-	network_mounts = NULL;
 	mounts = g_volume_monitor_get_mounts (volume_monitor);
 
 	for (l = mounts; l != NULL; l = l->next) {
@@ -799,6 +818,31 @@ update_places (NautilusPlacesSidebar *sidebar)
 	add_heading (sidebar, SECTION_NETWORK,
 		     _("Network"));
 
+	network_volumes = g_list_reverse (network_volumes);
+	for (l = network_volumes; l != NULL; l = l->next) {
+		volume = l->data;
+		mount = g_volume_get_mount (volume);
+
+		if (mount != NULL) {
+			network_mounts = g_list_prepend (network_mounts, mount);
+			continue;
+		} else {
+			icon = g_volume_get_icon (volume);
+			name = g_volume_get_name (volume);
+			tooltip = g_strdup_printf (_("Mount and open %s"), name);
+
+			add_place (sidebar, PLACES_MOUNTED_VOLUME,
+				   SECTION_NETWORK,
+				   name, icon, NULL,
+				   NULL, volume, NULL, 0, tooltip);
+			g_object_unref (icon);
+			g_free (name);
+			g_free (tooltip);
+		}
+	}
+
+	g_list_free_full (network_volumes, g_object_unref);
+
 	network_mounts = g_list_reverse (network_mounts);
 	for (l = network_mounts; l != NULL; l = l->next) {
 		mount = l->data;
@@ -2453,17 +2497,9 @@ find_prev_or_next_row (NautilusPlacesSidebar *sidebar,
 		       GtkTreeIter *iter,
 		       gboolean go_up)
 {
-	GtkTreeSelection *selection;
-	GtkTreeModel *model;
-	int place_type;
+	GtkTreeModel *model = GTK_TREE_MODEL (sidebar->store);
 	gboolean res;
-
-	selection = gtk_tree_view_get_selection (sidebar->tree_view);
-	res = gtk_tree_selection_get_selected (selection, &model, iter);
-
-	if (!res) {
-		goto out;
-	}
+	int place_type;
 
 	if (go_up) {
 		res = gtk_tree_model_iter_previous (model, iter);
@@ -2484,7 +2520,6 @@ find_prev_or_next_row (NautilusPlacesSidebar *sidebar,
 		}
 	}
 
- out:
 	return res;
 }
 
@@ -2535,6 +2570,30 @@ properties_cb (GtkMenuItem           *item,
 	gtk_tree_path_free (path);
 }
 
+static gboolean
+nautilus_places_sidebar_focus (GtkWidget *widget,
+			       GtkDirectionType direction)
+{
+	NautilusPlacesSidebar *sidebar = NAUTILUS_PLACES_SIDEBAR (widget);
+	GtkTreePath *path;
+	GtkTreeIter iter;
+	gboolean res;
+
+	res = get_selected_iter (sidebar, &iter);
+
+	if (!res) {
+		gtk_tree_model_get_iter_first (GTK_TREE_MODEL (sidebar->store), &iter);
+		res = find_next_row (sidebar, &iter);
+		if (res) {
+			path = gtk_tree_model_get_path (GTK_TREE_MODEL (sidebar->store), &iter);
+			gtk_tree_view_set_cursor (sidebar->tree_view, path, NULL, FALSE);
+			gtk_tree_path_free (path);
+		}
+	}
+
+	return GTK_WIDGET_CLASS (nautilus_places_sidebar_parent_class)->focus (widget, direction);
+}
+
 /* Handler for GtkWidget::key-press-event on the shortcuts list */
 static gboolean
 bookmarks_key_press_event_cb (GtkWidget             *widget,
@@ -2542,7 +2601,12 @@ bookmarks_key_press_event_cb (GtkWidget             *widget,
 			      NautilusPlacesSidebar *sidebar)
 {
   guint modifiers;
-  GtkTreeIter iter;
+  GtkTreeIter selected_iter;
+  GtkTreePath *path;
+
+  if (!get_selected_iter (sidebar, &selected_iter)) {
+	  return FALSE;
+  }
 
   modifiers = gtk_accelerator_get_default_mod_mask ();
 
@@ -2550,10 +2614,6 @@ bookmarks_key_press_event_cb (GtkWidget             *widget,
        event->keyval == GDK_KEY_KP_Enter ||
        event->keyval == GDK_KEY_ISO_Enter ||
        event->keyval == GDK_KEY_space)) {
-
-      GtkTreeModel *model;
-      GtkTreeSelection *selection;
-      GtkTreeIter iter;
       NautilusWindowOpenFlags flags = 0;
 
       if ((event->state & modifiers) == GDK_SHIFT_MASK) {
@@ -2562,12 +2622,8 @@ bookmarks_key_press_event_cb (GtkWidget             *widget,
           flags = NAUTILUS_WINDOW_OPEN_FLAG_NEW_WINDOW;
       }
 
-      model = gtk_tree_view_get_model (sidebar->tree_view);
-      selection = gtk_tree_view_get_selection (sidebar->tree_view);
-      gtk_tree_selection_get_selected (selection, NULL, &iter);
-
-      open_selected_bookmark (sidebar, model, &iter, flags);
-
+      open_selected_bookmark (sidebar, GTK_TREE_MODEL (sidebar->store),
+			      &selected_iter, flags);
       return TRUE;
   }
 
@@ -2577,17 +2633,19 @@ bookmarks_key_press_event_cb (GtkWidget             *widget,
   }
 
   if (event->keyval == GDK_KEY_Up) {
-      if (find_prev_row (sidebar, &iter)) {
-	      gtk_tree_selection_select_iter (gtk_tree_view_get_selection (sidebar->tree_view),
-					      &iter);
+      if (find_prev_row (sidebar, &selected_iter)) {
+	      path = gtk_tree_model_get_path (GTK_TREE_MODEL (sidebar->store), &selected_iter);
+	      gtk_tree_view_set_cursor (sidebar->tree_view, path, NULL, FALSE);
+	      gtk_tree_path_free (path);
       }
       return TRUE;
   }
 
   if (event->keyval == GDK_KEY_Down) {
-      if (find_next_row (sidebar, &iter)) {
-	      gtk_tree_selection_select_iter (gtk_tree_view_get_selection (sidebar->tree_view),
-					      &iter);
+      if (find_next_row (sidebar, &selected_iter)) {
+	      path = gtk_tree_model_get_path (GTK_TREE_MODEL (sidebar->store), &selected_iter);
+	      gtk_tree_view_set_cursor (sidebar->tree_view, path, NULL, FALSE);
+	      gtk_tree_path_free (path);
       }
       return TRUE;
   }
@@ -3430,6 +3488,7 @@ nautilus_places_sidebar_class_init (NautilusPlacesSidebarClass *class)
 	G_OBJECT_CLASS (class)->dispose = nautilus_places_sidebar_dispose;
 
 	GTK_WIDGET_CLASS (class)->style_set = nautilus_places_sidebar_style_set;
+	GTK_WIDGET_CLASS (class)->focus = nautilus_places_sidebar_focus;
 }
 
 static void
